#!/usr/bin/env python3
# Created by Yuki Isobe in May 2022
# Modified by Shinobu Ozaki in August 2022

import numpy as np
import argparse
import os
from astropy.io import fits
from scipy.stats import sigmaclip
from bias_overscan import bias_overscan
from bias_overscan import overscan
from cosmicrays import cosmicrays

def errestimate(infile, biasfile, rawdatadir='', overwrite=False):
    print('\n#####################')
    print('Error estimating.')
    inhdl = fits.open(infile)
    inhdr = inhdl[0].header
    basename = inhdr['FRAMEID']
    ername = basename + '.er.fits'
    if os.path.isfile(ername):
        if not overwrite:
            print('\t Error frame already exists. ' + ername)
            print('\t This procedure is skipped.')
            inhdl.close()
            return ername, True
    NAXIS1 = inhdr['NAXIS1']
    NAXIS2 = inhdr['NAXIS2']
    img = inhdl[0].data

    # Readout noise from bias frame
    ovbiasname, stat = bias_overscan(biasfile, rawdatadir=rawdatadir, overwrite=overwrite)
    if stat == False:
        return ovbiasname, False

    # readout noise
    biashdl = fits.open(ovbiasname)
    biasimg = biashdl[0].data

    binfac1 = biashdl[0].header['BIN-FCT1']
    ovs = overscan[binfac1]

    marg = []
    for i in range(len(ovs)):
        marg.append(ovs[i,3] - ovs[i,2] + 1)

    # Set the start X of right half image
    # CCD gap: 5 arcsec
    # pixel scale: 0.104 arcsec/pix
    gap_width = int(5 / 0.104 / binfac1)

    xs = 0
    na1 = 0
    for i in range(len(marg)):
        xe = xs + marg[i]
        na1 +=  xe -xs
    na1 += gap_width
    if na1 != NAXIS1:
        print('Error: bias format might be different from science data format.')
        print('NAXIS1: {} in bias image, {} in science data.'.format(na1, NAXIS1))
        return ername, False

    rnoise_data = np.zeros([NAXIS2, na1], dtype=np.float32)
    
    xs = 0
    nsig = 5.0  # sigma clipe with 5-sigma
    stddev = []
    for i in range(len(marg)):
        xe = xs + marg[i]
        clipped_data, low, upp = sigmaclip(biasimg[:,xs:xe], nsig, nsig)
        tempstd = np.std(clipped_data)
        rnoise_data[:,xs:xe] = tempstd
        stddev.append(tempstd)
        xs = xe
        if i == 3:
            xs += gap_width

    print('Readout noise for leftmost to rightmost readout area (electrons):')
    print(stddev)

    # variance of photon noise
    np.place(img, img < 0., 0)
    
    # Adding the noises
    noise_data = np.sqrt(img + rnoise_data ** 2)
    
    # Creating HDU data
    outhdu = fits.PrimaryHDU(data = noise_data)
    outhdl = fits.HDUList([outhdu])
    outhdl[0].header = inhdr
    outhdl.writeto(ername, overwrite=overwrite)
    inhdl.close()
    biashdl.close()
    outhdl.close()
    print('\t Created error file: '+ername)

    return ername, True

if __name__ == '__main__':
    parser=argparse.ArgumentParser(description='This is the script for error estimation.')
    parser.add_argument('infile', help='Input FITS file')
    parser.add_argument('biasfile', help='Bias FITS file')
    parser.add_argument('-d', help='Raw data directroy', dest='rawdatadir',
                        action='store', default='')
    parser.add_argument('-o', help='Overwrite flag', dest='overwrite', action='store_true', default=False)
    args = parser.parse_args()
    errestimate(args.infile, biasfile=args.biasfile, rawdatadir=args.rawdatadir, overwrite=args.overwrite)

