#!/usr/bin/env python3
#
# <input fits> <output fits> <stat lambda> <end lambda>
#

from astropy.io import fits
import numpy as np
import argparse


def Wav2Pix(w, crpix, crval, cdelt):
    # w: wavelength (A)
    # crpix: reference pixel
    # crval: reference wavelength (A)
    # cdelt: waverength step per pixel (A)
    pix = (w - crval) / cdelt + crpix - 1.0
    return pix


def MonoImage(hdl, band, en):
    # hdl: HDU list object of astropy
    # band: wavelength range for integration
    # en: extension number in which the data is stored.
    
    scidata = hdl[en].data
    hdr = hdl[en].header
    if 'CD3_3' in hdr:
        cdelt = hdr['CD3_3']
    else:
        cdelt = hdr['CDELT3']
    crpix = hdr['CRPIX3']
    crval = hdr['CRVAL3']

    pix1 = Wav2Pix(band[0], crpix, crval, cdelt)
    pix2 = Wav2Pix(band[1], crpix, crval, cdelt)

    print(pix1, band[0])
    print(pix2, band[1])

    ipix1 = int(pix1)
    ipix2 = int(pix2)

    imdata = np.sum(scidata[ipix1+1:ipix2+1,:,:], axis=0)

    imdata += scidata[ipix1,:,:] * (pix1 - ipix1)
    imdata += scidata[ipix2+1,:,:] * (pix2 - ipix2)
    
    return imdata * cdelt, [pix1, pix2]


def LineImage(incube, outfits, onband, c1, c2, en):
    # incube: input data cube FITS file name
    # outfits: output FITS file name
    # onband: target wavelength range
    # c1: continuum range 1
    # c2: continuum range 2
    # en: extension number in which the data is stored.
    
    hdl = fits.open(incube)
    ondata, onpix = MonoImage(hdl, onband, en)
    if c1 is not None:
        con1data, con1pix = MonoImage(hdl, c1, en)
        con1data /= (con1pix[1] - con1pix[0])  ## flux/ 
        if c2 is not None:
            con2data, con2pix = MonoImage(hdl, c2, en)
            con2data /= (con2pix[1] - con2pix[0])

            # creating the continuum image at the onband center.
            c_onpix = (onpix[0] + onpix[1]) / 2.0
            c_con1pix = (con1pix[0] + con1pix[1]) / 2.0
            c_con2pix = (con2pix[0] + con2pix[1]) / 2.0

            w1 = (c_onpix - c_con1pix) / (c_con2pix - c_con1pix)
            w2 = (c_con2pix - c_onpix) / (c_con2pix - c_con1pix)

            print(c_onpix, " ", c_con1pix, " ", c_con2pix)
            print(w1, " ", w2)
            print(con1data)
            print(con2data)
            con1data = w2 * con1data + w1 * con2data

        linedata = ondata - con1data * (onpix[1] - onpix[0])
    else:
        linedata = ondata
        
    outhdu = fits.PrimaryHDU(data=linedata)
    outhdl = fits.HDUList([outhdu])
    hdr = hdl[en].header
    hdr['WCSDIM']=2

    remove_key = ['CD3_3','CD1_3', 'CD2_3', 'CD3_1', 'CD3_2', 'CDELT3',
                  'CRPIX3','CRVAL3','CUNIT3','CTYPE3', 'LTM3_3',
                  'WAT3_001', 'NAXIS3', 'XTENSION', 'EXTNAME',
                  'HDUCLAS1', 'HDUCLAS2', 'HDUVERS', 'HDUDOC', 'HDUCLASS',
                  'PCOUNT', 'GCOUNT', 'ERRDATA']
    
    for key in remove_key:
        if key in hdr:
            hdr.remove(key)
    outhdl[0].header = hdr
    outhdl.writeto(outfits)
    outhdl.close()
    hdl.close()
    return

    
if __name__ == '__main__':
    parser=argparse.ArgumentParser(description=
                '')
    parser.add_argument('incube', help='Input data cube')
    parser.add_argument('outfits', help='Output fits name')
    parser.add_argument('onband', nargs=2, type=float,
                        help='Onband wavelength range (A)')
    parser.add_argument('-c1', '--continuum1', nargs=2, type=float,
                        help='Continuum1 wavelength range (A)')
    parser.add_argument('-c2', '--continuum2', nargs=2,  type=float,
                        help='Continuum1 wavelength range (A)')
    parser.add_argument('-en', '--extnum', type=int, default=0,
                        help='FITS extension number to be used. Dfault is 0.')
    args = parser.parse_args()

    LineImage(args.incube, args.outfits, args.onband, args.continuum1,
              args.continuum2, args.extnum)

