#!/usr/bin/env python
# python3 OK

import sys
import argparse
from bias_overscan import bias_overscan
from cosmicrays import cosmicrays
from mkchimage import mkchimage
from flatfielding import flatfielding
from transform import transform
from fluxcalib import fluxcalib
from mkcube import mkcube
from skysub import skysub
from skysub_old import skysub_old
from zero_padding import zero_padding
from astropy.io import fits


def object(infile, domeflat=None, domecomp=None, calflat=None,
           comparison=None, stdstar=None, rawdatadir='', overwrite=False,
           ql=False, bo=False):
    print('\n#############################')
    print('#############################')
    print('Reducing the object frame')

    basenames=['','']
    # bias subtraction, overscan region removing,
    # bad pixel retoring, hedear correction
    ovname, stat = bias_overscan(infile, rawdatadir=rawdatadir,
                                 overwrite=overwrite)
    if stat == False:
        return ovname, False

    # cosmicray removing
    crname, maskname, stat = cosmicrays(ovname, overwrite=overwrite)
    if stat == False:
        return crname, False
    
    # flat fielding
    if domecomp is not None:
        bias_overscan(domecomp+'.fits', rawdatadir=rawdatadir,
                      overwrite=overwrite)
    ffname, stat = flatfielding(crname, domeflat, calflat, domecomp=domecomp,
                                comp=comparison, overwrite=overwrite, ql=ql)
    if stat == False:
        return ffname, False

    # extraction and making each channel image
    basename, stat = mkchimage(ffname, calflat, overwrite=overwrite)
    if stat == False:
        return basename, False

    # Transorming
    stat = transform(basename, comparison, calflat, overwrite=overwrite)
    if stat == False:
        return '', False
            
    # Flux calibration
    if stdstar is not None and stdstar != 'None':
        fcfile, stat = fluxcalib(basename, stdstar+'.sens.fits', \
                                 overwrite=overwrite)
        postfix = 'fc'
        if stat == False:
            return fcfile, False
    else:
        postfix = 'wc'
            
    # Making data cube
    cubefile, stat = mkcube(basename, postfix, overwrite=overwrite)
    if stat == False:
        return cubefile, False

    # Sky subtraction
    obsdate_s = fits.getval(cubefile, 'DATE-OBS')
    obsdate = [int(a) for a in obsdate_s.split('-')]

    if obsdate[0] >  2021 or (obsdate[0] ==  2021 and obsdate[1] >= 8):
        skysubedfile, stat = skysub(cubefile, comparison,
                                    overwrite=overwrite, ql=ql)
    else:
        skysubedfile, stat = skysub_old(cubefile, comparison,
                                        overwrite=overwrite, ql=ql)
        
    if stat == False:
        return skysubedfile, False

    # Padding with NaN
    ans = 'n'
    if not ql:
        if sys.version_info[0] == 3:
            ans = input("Do you want to pad 0-value around the spectrum edges with NaN? [y/n] ")
        else:
            ans = raw_input("Do you want to pad 0-value around the spectrum edges with NaN? [y/n] ")

    if ans is 'y':
        zero_padding(skysubedfile)
        
    return skysubedfile, True

if __name__ == '__main__':
    # Analize argments
    parser=argparse.ArgumentParser(description='This is the reduction'+\
                                   'script for an object frame.')
    parser.add_argument('infile', help='Input FITS file')
    parser.add_argument('dflatid', help='Domeflat frame ID')
    parser.add_argument('cflatid', help='Calflat frame ID')
    parser.add_argument('compid', help='Comparison frame ID')
    parser.add_argument('-domecompid', help='Comparison frame ID for domeflat', default=None)
    parser.add_argument('-stdstarid', help='Standard star frame ID', default=None)
    parser.add_argument('-d', help='Raw data directroy', dest='rawdatadir', 
                        action='store', default='')
    parser.add_argument('-o', help='Overwrite flag', dest='overwrite', 
                        action='store_true',default=False)
    parser.add_argument('-ql', help='Quick look mode', dest='ql',
                        action='store_true',default=False)
    parser.add_argument('-bo', help='Bias template is made from an overscan region. In default, the bias template file created by mkbiastemplate.py is use.',\
                        action='store_true', default=False)
    args = parser.parse_args()
    
    object(args.infile, domeflat=args.dflatid, domecomp=args.domecompid,
           calflat=args.cflatid, comparison=args.compid, stdstar = args.stdstarid,
           rawdatadir = args.rawdatadir, overwrite = args.overwrite,
           ql=args.ql, bo=args.bo)
