#!/usr/bin/env python


from astropy.io import fits
from astropy.wcs import WCS, utils
import numpy as np
from scipy.interpolate import interp2d
import argparse


def scalematch(infile, outfile, kind='linear'):
    # infile: input data cube FITS file name
    # outfile: output FITS file name
    # kind: linear, cubic, quintic
    
    hdl = fits.open(infile)
    data = hdl[0].data
    
    # calculating scaling factor form WCS
    hdr = hdl[0].header
    wcs = WCS(hdr)
    ps = utils.proj_plane_pixel_scales(wcs)
    fac = ps[1] / ps[0]
    
    # Resamplin
    x = np.arange(data.shape[2])*ps[0]
    y = np.arange(data.shape[1])*ps[1]
    ynew = np.arange(int((data.shape[1]-1)*fac)+1)*ps[0]
    newdata = np.zeros([data.shape[0], len(ynew), data.shape[2]], dtype=np.float32)
    for i in range(data.shape[0]):
        f = interp2d(x, y, data[i,:,:], kind=kind)
        newdata[i,:,:] = f(x, ynew)/fac  # for flux conservation
    
    # Modifying FITS header
    outhdr = hdl[0].header
    outhdr['CD1_2'] = outhdr['CD1_2']/fac
    outhdr['CD2_2'] = outhdr['CD2_2']/fac
    outhdr['CRPIX2'] = (outhdr['CRPIX2'] - 1) * fac + 1
    
    # Creating FITS file
    outhdu = fits.PrimaryHDU(data=newdata, header=outhdr)
    outhdl = fits.HDUList([outhdu])
    outhdl.writeto(outfile)
    outhdl.close()
    hdl.close()
    return

    
if __name__ == '__main__':
    desc = 'This is the script for matching pixel scales along both axes by resampling'
    parser=argparse.ArgumentParser(description=desc)
    parser.add_argument('infile', help='Input cube FITS file')
    parser.add_argument('outfile', help='Output FITS file')
    parser.add_argument('-k', '--kind',
                        help='Kind of interpolation. Default is \'linear\'',
                        choices=['linear', 'cubic', 'quintic'],
                        default='linear')
    args = parser.parse_args()

    scalematch(args.infile, args.outfile, kind=args.kind)

